;Written by Josh Howard
;www.houseofbim.com

(defun HOB_Remove (/ v)
  (if (get_tile "wlst")
    (foreach x (reverse(read (strcat "(" (get_tile "wlst") ")")))
      (setq wls (acet-list-remove-nth x wls))))
  (HOB_LoadLists)
  )

(defun HOB_Add (/ mat ser)
  (setq mat (nth (atoi (get_tile "mlst")) materials))
  (foreach x (read (strcat "(" (get_tile "slst") ")"))
    (setq ser (nth x sls))
    (if (null(assoc ser wls))
      (setq wls (append wls (list (list ser mat)))))
    )
  (HOB_LoadLists)
  )

(defun HOB_Apply ()
  (done_dialog)
  (foreach x wls
    (foreach y lst
      (if (= (car y) (car x))
	(foreach z (cdr y)
	  (vla-put-material (vlax-ename->vla-object z) (nth 1 x)))
	)
      )
    )  
  (princ)
  )

(defun HOB_Reset ()
  (done_dialog)  
  (foreach x lst
    (foreach y (cdr x)
      (vla-put-material (vlax-ename->vla-object y) "ByLayer")))
  )

(defun HOB_GetMaterials (DoImport / ret pth)
  (if DoImport
    (if (setq pth (getfiled "Select a Material DWG" "" "dwg" 0))
      (vla-delete(vla-insertblock(vla-get-modelspace(vla-get-activedocument(vlax-get-acad-object)))
		   (vlax-3d-point (list 0 0 0)) pth 1.0 1.0 1.0 0.0))))
  (vlax-for x (vla-get-materials(vla-get-activedocument(vlax-get-acad-object)))
    (setq ret (append ret (list (vla-get-name x)))))
  ret
  )

(defun HOB_Create_Dialog (/ ret fil)
  (setq pth (vl-filename-mktemp "hobmats.dcl")
	fil (open pth "w"))
  (write-line "HOBmats : dialog {" fil)
  (write-line "  label = \"Select Pairs of Services & Materials to Process\"; width=100;"  fil)
  (write-line "    :column {" fil)
  (write-line "      :row {" fil)
  (write-line "        :column {" fil)
  (write-line "          :button { label=\"Reset All Materials\"; key=\"Reset\"; width=50; alignment=left; } }" fil)
  (write-line "        :column {" fil)
  (write-line "          :button { label=\"Import Materials\"; key=\"Import\"; width=50; alignment=right; } } }" fil)
  (write-line "      :row {" fil)
  (write-line "        :boxed_column { label=\"Select Service\";" fil )
  (write-line "          :list_box { key=\"slst\"; height=15; multiple_select=true; } }" fil)
  (write-line "        :boxed_column { label=\"Select Material\";" fil)
  (write-line "          :list_box { key=\"mlst\"; height=15; multiple_select=false; } } }" fil)
  (write-line "      :row {" fil)
  (write-line "        :column {" fil)
  (write-line "          :button { label=\"Remove Selected\"; key=\"Remove\"; width=50; alignment=left; } }" fil)
  (write-line "        :column {" fil)
  (write-line "          :button { label=\"Add Service/Material Pair\"; key=\"Add\"; width=50; alignment=right; } } }" fil)
  (write-line "      :row {" fil)
  (write-line "        :boxed_column { label=\"Queued Pairs for Processing\";" fil)
  (write-line "          :list_box { key=\"wlst\"; height=15; multiple_select=true; } } } }" fil)
  (write-line "  ok_only ; }" fil)
  (close fil)
  pth
  )

(defun HOB_LoadLists (/ cs cm)
  (setq cs (if (get_tile "slst") (get_tile "slst") "0")
	cm (if (get_tile "mlst") (get_tile "mlst") "0"))
  (start_list "slst") (mapcar 'add_list sls) (end_list) 
  (start_list "mlst") (mapcar 'add_list materials) (end_list) 
  (if (>= (length wls) 1)
    (progn
      (start_list "wlst") (mapcar '(lambda (x) (add_list(strcat (car x) "" (cadr x)))) wls) (end_list))
    (progn
      (start_list "wlst") (end_list))    
    )
  (set_tile "slst" cs)
  (set_tile "mlst" cm)
  )

(defun HOB_ShowDialog ()
  (if (not (new_dialog "HOBmats" dclDialog)) (exit))
  (HOB_LoadLists)
  (action_tile "Reset" "(HOB_Reset)")
  (action_tile "Import" "(setq materials (HOB_GetMaterials t))(HOB_LoadLists)")
  (action_tile "Remove" "(HOB_Remove)")
  (action_tile "Add" "(HOB_Add)")
  (action_tile "accept" "(HOB_Apply)")  
  (start_dialog)
  (unload_dialog dclDialog)
  )

(defun C:HOBMaterialMapper (/ ss cc path ee oo sn itm lst sls wls materials dclDialog)
  (setq ss (ssget "_X")
	cc 0
	path (HOB_Create_Dialog))


  (repeat (sslength ss)
    (setq ee (ssname ss cc)
	  oo (vlax-ename->vla-object ee)
	  cc (+ cc 1))
    (cond
      ((vlax-property-available-p oo 'ServiceName)	;Fabrication
       (setq sn (vlax-get-property oo 'ServiceName)))
      ((vlax-property-available-p oo 'Service)		;Plant 3D
       (setq sn (vl-catch-all-apply 'vlax-get-property (list oo "Service"))))
      )
    (if (and (= (type sn) 'STR) (/= sn ""))
      (if (assoc sn lst)
	(setq itm (assoc sn lst)
	      itm (append itm (list ee))
	      lst (subst itm (assoc sn lst) lst))
	(setq itm (list sn ee)
	      sls (append sls (list sn))
	      lst (append lst (list itm)))
	)
      )
    )
  (setq materials (HOB_GetMaterials nil)
	dclDialog (load_dialog path))  
  
  (HOB_ShowDialog)  
  
  (if (findfile path) (vl-file-delete path))
  (princ)
  )


